<?php
/*

Simple class to let themes add dependencies on plugins in ways they might find useful

Example usage:

$test = new Theme_Plugin_Dependency( 'plugin-slug');
if ( $test->check_active() )
    echo 'Plugin is installed and activated!';
else if ( $test->check() )
    echo 'Plugin is installed, but not activated. <a href="'.$test->activate_link().'">Click here to activate the plugin.</a>';
else if ( $install_link = $test->install_link() )
    echo 'Plugin is not installed. <a href="'.$install_link.'">Click here to install the plugin.</a>';
else
    echo 'Plugin is not installed and could not be found in the Plugin Directory. Please install this plugin manually.';

*/
if (!class_exists('weaverii_Load_Plugin')) {
    class weaverii_Load_Plugin {
	// input information from the theme
	var $name;
        var $slug;
        var $filename;
	var $version;

        // both slug and PluginURI are required for checking things
        function __construct( $slug ) {
	    $this->name = '';
            $this->slug = $slug;
            $this->filename = '';
            if ( empty( $this->plugins ) )
                $plugins = get_plugins();

	    foreach ($plugins as $plugin => $val) {
		$last = strrpos($plugin, '/');
		if ($last !== false)
		    $pslug = substr($plugin, 0, $last);
		else
		    $pslug = '';

		if ($pslug == $slug) {	// found it - so it is installed!
		    $this->name = $val['Name'];
		    $this->filename = $plugin;
		    $this->version = $val['Version'];
                    //  echo '<br /> filename,version,slug: ' . $this->filename . ' - ' . $this->version . ' - ' . $this->slug . '<br />';
		}
	    }
        }

        // return true if installed, false if not
        function check() {
            return $this->filename != '';
        }

	// return version of plugin
        function version() {
            return $this->version;
        }

        // return true if installed and activated, false if not
        function check_active() {
            $plugin_file = $this->get_plugin_file();
            if ($plugin_file) return is_plugin_active($plugin_file);
            return false;
        }

        // gives a link to activate the plugin
        function activate_link() {
            $plugin_file = $this->get_plugin_file();
            if ($plugin_file) return wp_nonce_url(self_admin_url('plugins.php?action=activate&plugin='.$plugin_file), 'activate-plugin_'.$plugin_file);
            return false;
        }

        // return a nonced installation link for the plugin. checks wordpress.org to make sure it's there first.
        function install_link() {
            include_once ABSPATH . 'wp-admin/includes/plugin-install.php';

            $info = plugins_api('plugin_information', array('slug' => $this->slug ));

            if ( is_wp_error( $info ) )
                return false; // plugin not available from wordpress.org

            return wp_nonce_url(self_admin_url('update.php?action=install-plugin&plugin=' . $this->slug), 'install-plugin_' . $this->slug);
        }

        // return array key of plugin if installed, false if not, private because this isn't needed for themes, generally
        private function get_plugin_file() {
            return $this->filename;
        }
    }
}
?>
